require 'rubygems'
require 'bundler/setup'
require 'httparty'
require_relative 'onebusaway_ruby_data'

class OneBusAway
  include HTTParty
  
  def initialize(key, url)
    raise "invalid url format" unless url =~ URI::regexp
    self.class.base_uri(url)
    @api_key = key
    raise "cannot connect" unless valid_connection?
  end
  
  def valid_connection?
    begin
      response = get("/current-time.json?key=#{@api_key}")
      true
    rescue
      return false
    end
  end
  
  def get(url)
    response = self.class.get(url)
    if response.success?
      raise response['text'] unless response['code'] == 200
      response
    else
      raise response.response
    end
  end
  
  # lists all supported agencies along with the center of their coverage area
  # @return [Hash<String, Agency>] a hash of all agencies with their agency ids as keys
  def agencies_with_coverage
    response = get("/agencies-with-coverage.json?key=#{@api_key}")
    response['data'].reduce({}) do |hash, agency|
      a = Agency.new.extend(AgencyRepresenter).from_json(agency['agency'].to_json)
      hash[a.agency_id] = a; hash
    end
  end
  
  # gets details for a specific agency
  # @return [Agency]
  # @param [Hash] opts
  # @option opts [String] :agency_id the id of the agency to be returned
  def agency(opts)
    raise if opts[:agency_id].nil?
    response = get("/agency/#{opts[:agency_id]}.json?key=#{@api_key}")
    Agency.new.extend(AgencyRepresenter).from_json(response['data']['entry'].to_json)
  end
  
  # arrival_and_departure_for_stop - details about a specific arrival/departure at a stop
  def arrival_and_departure_for_stop(stop_id, trip_id, service_date)
    response = get("/arrival-and-departure-for-stop/#{stop_id}.json?key=#{@api_key}&tripId=#{trip_id}&serviceDate=#{service_date}")
  end
  
  # arrivals_and_departures_for_stop - get current arrivals and departures for a stop
  def arrivals_and_departures_for_stop(stop_id)
    response = get("/arrivals-and-departures-for-stop/#{stop_id}.json?key=#{@api_key}")
    response['data']['arrivalsAndDepartures'].reduce({}) do |hash, ad|
      r = ArrivalAndDeparture.new.extend(ArrivalAndDepartureRepresenter).from_json(ad.to_json)
      hash[r.route_id] = r
      hash
    end
  end
  
  # cancel_alarm - cancel a registered alarm
  def cancel_alarm(alarm_id)
    response = get("/cancel-alarm/#{alarm_id}.json?key=#{@api_key}")
  end
  
  # retrieves the current system time
  #
  # @return [DateTime]
  def current_time
    response = get("/current-time.json?key=#{@api_key}")
    Time.at(response["data"]["time"] / 1000).to_datetime
  end
  
  # register_alarm_for_arrival_and_departure_at_stop - register an alarm for an arrival-departure event
  def register_alarm_for_arrival_and_departure_at_stop(stop_id, trip_id, service_date, vehicle_id, stop_sequence, alarm_time_offset, callback_url)
    response = get("/register-alarm-for-arrival-and-departure-at-stop/#{stop_id}.json?key=#{@api_key}&tripId=#{trip_id}&serviceDate=#{service_date}&vehicleId=#{vehicle_id}&stopSequence=#{stop_sequence}&alarmTimeOffset=#{alarm_time_offset}&url=#{callback_url}")
  end
  
  # route_ids_for_agency - get a list of all route ids for an agency
  def route_ids_for_agency(agency)
    response = get("/route-ids-for-agency/#{agency}.json?key=#{@api_key}")
    response["data"]["list"] 
  end
  
  # route - get details for a specific route
  def route(route_id)
    response = get("/route/#{route_id}.json?key=#{@api_key}")
    Route.new.extend(RouteRepresenter).from_json(response['data'].to_json)
  end
  
  # routes_for_agency - get a list of all routes for an agency
  def routes_for_agency(agency)
    response = get("/routes-for-agency/#{agency}.json?key=#{@api_key}")
    response['data']['list'].reduce({}) do |hash, route|
      r = Route.new.extend(RouteRepresenter).from_json(route.to_json)
      hash[r.route_id] = r
      hash
    end
  end
  
  # routes_for_location - search for routes near a location, optionally by route name
  def routes_for_location(lat, lon)
    response = get("/routes-for-location.json?key=#{@api_key}&lat=#{lat}&lon=#{lon}")
    response['data']['routes'].reduce({}) do |hash, route|
      r = Route.new.extend(RouteRepresenter).from_json(route.to_json)
      hash[r.route_id] = r
      hash
    end
  end
  
  # schedule_for_stop - get the full schedule for a stop on a particular day
  def schedule_for_stop(stop)
    response = get("/schedule-for-stop/#{stop}.json?key=#{@api_key}")
    StopSchedule.new.extend(StopScheduleRepresenter).from_json(response['data']['entry'].to_json)
  end
  
  # shape - get details for a specific shape (polyline drawn on a map)
  def shape(shape_id)
    response = get("/shape/#{shape_id}.json?key=#{@api_key}")
  end
  
  # stop_ids_for_agency - get a list of all stops for an agency
  def stop_ids_for_agency(agency)
    response = get("/stop-ids-for-agency/#{agency}.json?key=#{@api_key}")
    response["data"]["list"]
  end
  
  # stop - get details for a specific stop
  def stop(stop_id)
    response = get("/stop/#{stop_id}.json?key=#{@api_key}")
    Stop.new.extend(StopRepresenter).from_json(response['data'].to_json)
  end
  
  # stops_for_location - search for stops near a location, optionally by stop code
  def stops_for_location(lat, lon)
    response = get("/stops-for-location.json?key=#{@api_key}&lat=#{lat}&lon=#{lon}")
    response['data']['stops'].reduce({}) do |hash, stop|
      s = Stop.new.extend(StopRepresenter).from_json(stop.to_json)
      hash[s.stop_id] = s
      hash
    end
  end
  
  # stops_for_route - get the set of stops and paths of travel for a particular route
  def stops_for_route(route_id)
    response = get("/stops-for-route/#{route_id}.json?key=#{@api_key}&version=2")
    response['data']['references']['stops'].reduce({}) do |hash, stop|
      s = Stop.new.extend(StopRepresenter).from_json(stop.to_json)
      hash[s.stop_id] = s
      hash
    end
  end
  
  # trip_details - get extended details for a specific trip
  def trip_details(trip_id)
    response = get("/trip-details/#{trip_id}.json?key=#{@api_key}")
  end
  
  # trip_for_vehicle - get extended trip details for current trip of a specific transit vehicle
  def trip_for_vehicle(vehicle_id)
    response = get("/trip-for-vehicle/#{vehicle_id}.json?key=#{@api_key}")
  end
  
  # trip - get details for a specific trip
  def trip(trip_id)
    response = get("/trip/#{trip_id}.json?key=#{@api_key}")
  end
  
  # trips_for_location - get active trips near a location
  def trips_for_location(lon, lat)
    response = get("/trips-for-location.json?key=#{@api_key}&lat=#{lat}&lon=#{lon}")
  end
  
  # trips_for_route - get active trips for a route
  def trips_for_route(route_id)
    response = get("/trips-for-route/#{route_id}.json?key=#{@api_key}")
  end
  
  # vehicles_for_agency - get active vehicles for an agency
  def vehicles_for_agency(agency_id)
    response = get("/vehicles-for-agency/#{agency_id}.json?key=#{@api_key}")
    response['data']['list'].reduce({}) do |hash, vehicle|
      v = Vehicle.new.extend(VehicleRepresenter).from_json(vehicle.to_json)
      hash[v.vehicle_id] = v
      hash
    end
  end
  
  private :get, :valid_connection?
end